from setuptools import setup, find_packages

setup(name="ZeroMqEventManager",
      version="1.0.1",
      packages=find_packages(where="."),
      py_modules=["main"],
      install_requires=['ZeroMqMessageHandler==1.0.0',
                        'EdpHelper==0.15',
                        'MessageProcessorUtil==1.0.0'],
      url="http://www.e-deploy.com.br",
      author="Eduardo Almeida",
      author_email="eduardo.almeida@e-deploy.com.br")
